@@ -1,13 +1,14 @@
+from __future__  import absolute_import
 import sys
 import re
 import os
 
-from errors import DynamicAddressException
+from ciscoconfparse.errors import DynamicAddressException
 
-from ccp_util import _IPV6_REGEX_STR_COMPRESSED1, _IPV6_REGEX_STR_COMPRESSED2
-from ccp_util import _IPV6_REGEX_STR_COMPRESSED3
-from ccp_util import CiscoRange, IPv4Obj, IPv6Obj
-from ccp_abc import BaseCfgLine
+from ciscoconfparse.ccp_util import _IPV6_REGEX_STR_COMPRESSED1, _IPV6_REGEX_STR_COMPRESSED2
+from ciscoconfparse.ccp_util import _IPV6_REGEX_STR_COMPRESSED3
+from ciscoconfparse.ccp_util import CiscoRange, IPv4Obj, IPv6Obj
+from ciscoconfparse.ccp_abc import BaseCfgLine
 
 ### HUGE UGLY WARNING:
 ###   Anything in models_nxos.py could change at any time, until I remove this
@@ -17,7 +18,7 @@ from ccp_abc import BaseCfgLine
 ###
 ###   You have been warned :-)
 """ models_nxos.py - Parse, Query, Build, and Modify IOS-style configurations
-     Copyright (C) 2016-2018 David Michael Pennington
+     Copyright (C) 2016-2019 David Michael Pennington
 
      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
@@ -40,6 +41,7 @@ from ccp_abc import BaseCfgLine
 ##-------------  IOS Configuration line object
 ##
 
+MAX_VLAN = 4094
 
 class NXOSCfgLine(BaseCfgLine):
     """An object for a parsed IOS-style configuration line.  
@@ -87,27 +89,24 @@ class NXOSCfgLine(BaseCfgLine):
         This example illustrates use of the method.
 
         .. code-block:: python
-           :emphasize-lines: 17,20
+           :emphasize-lines: 17
 
            >>> config = [
            ...     '!',
-           ...     'interface Serial1/0',
-           ...     ' ip address 1.1.1.1 255.255.255.252',
+           ...     'interface Ethernet1/1',
+           ...     ' ip address 1.1.1.1/30',
            ...     '!',
-           ...     'interface ATM2/0',
+           ...     'interface Ethernet1/2',
            ...     ' no ip address',
            ...     '!',
-           ...     'interface ATM2/0.100 point-to-point',
-           ...     ' ip address 1.1.1.5 255.255.255.252',
-           ...     ' pvc 0/100',
-           ...     '  vbr-nrt 704 704',
+           ...     'interface Ethernet1/3',
+           ...     ' ip address 1.1.1.5/30',
+           ...     ' ip dhcp relay address 172.16.1.12'
+           ...     ' ip dhcp relay address 172.19.200.84',
            ...     '!',
            ...     ]
            >>> parse = CiscoConfParse(config)
-           >>> obj = parse.find_objects('^interface\sSerial')[0]
-           >>> obj.is_intf
-           True
-           >>> obj = parse.find_objects('^interface\sATM')[0]
+           >>> obj = parse.find_objects('^interface\sEthernet')[0]
            >>> obj.is_intf
            True
            >>>
@@ -129,27 +128,24 @@ class NXOSCfgLine(BaseCfgLine):
         This example illustrates use of the method.
 
         .. code-block:: python
-           :emphasize-lines: 17,20
+           :emphasize-lines: 17
 
            >>> config = [
            ...     '!',
-           ...     'interface Serial1/0',
-           ...     ' ip address 1.1.1.1 255.255.255.252',
+           ...     'interface Ethernet1/1',
+           ...     ' ip address 1.1.1.1/30',
            ...     '!',
-           ...     'interface ATM2/0',
+           ...     'interface Ethernet1/2',
            ...     ' no ip address',
            ...     '!',
-           ...     'interface ATM2/0.100 point-to-point',
-           ...     ' ip address 1.1.1.5 255.255.255.252',
-           ...     ' pvc 0/100',
-           ...     '  vbr-nrt 704 704',
+           ...     'interface Ethernet1/3.100',
+           ...     ' ip address 1.1.1.5/30',
+           ...     ' ip dhcp relay address 172.16.1.12'
+           ...     ' ip dhcp relay address 172.19.200.84',
            ...     '!',
            ...     ]
            >>> parse = CiscoConfParse(config)
-           >>> obj = parse.find_objects('^interface\sSerial')[0]
-           >>> obj.is_subintf
-           False
-           >>> obj = parse.find_objects('^interface\sATM')[0]
+           >>> obj = parse.find_objects('^interface\sEthernet')[-1]
            >>> obj.is_subintf
            True
            >>>
@@ -183,7 +179,7 @@ class NXOSCfgLine(BaseCfgLine):
 
            >>> config = [
            ...     '!',
-           ...     'interface FastEthernet1/0',
+           ...     'interface Ethernet1/1',
            ...     ' ip address 1.1.1.1 255.255.255.252',
            ...     '!',
            ...     'interface Loopback0',
@@ -191,7 +187,7 @@ class NXOSCfgLine(BaseCfgLine):
            ...     '!',
            ...     ]
            >>> parse = CiscoConfParse(config)
-           >>> obj = parse.find_objects('^interface\sFast')[0]
+           >>> obj = parse.find_objects('^interface\sEthernet')[0]
            >>> obj.is_loopback_intf
            False
            >>> obj = parse.find_objects('^interface\sLoop')[0]
@@ -248,7 +244,7 @@ class NXOSCfgLine(BaseCfgLine):
         return False
 
     @property
-    def in_portchannel(self):
+    def intf_in_portchannel(self):
         """Return a boolean indicating whether this port is configured in a port-channel
 
         """
@@ -267,20 +263,27 @@ urn -1 if it's not configured in a port-channel
         return retval
 
     @property
-    def is_portchannel(self):
+    def is_portchannel_intf(self):
         """Return a boolean indicating whether this port is a port-channel intf
 
         """
         return ('channel' in self.name.lower())
 
+    @property
+    def is_vpc_peerlink(self):
+        """Return a boolean indicating whether this port is configured as a vpc peer-link port"""
+        retval = self.re_match_iter_typed(
+            r'^\s*vpc\s+peer-link', result_type=str, default='')
+        return retval
+
 
 ##
-##-------------  IOS Interface ABC
+##-------------  NXOS Interface ABC
 ##
 
 # Valid method name substitutions:
 #    switchport -> switch
-#    spanningtree -> stp
+#    spanning-tree -> stp
 #    interfce -> intf
 #    address -> addr
 #    default -> def
@@ -479,7 +482,7 @@ class BaseNXOSIntfLine(NXOSCfgLine):
 
     @property
     def ordinal_list(self):
-        """Return a tuple of numbers representing card, slot, port for this interface.  If you call ordinal_list on GigabitEthernet2/25.100, you'll get this python tuple of integers: (2, 25).  If you call ordinal_list on GigabitEthernet2/0/25.100 you'll get this python list of integers: (2, 0, 25).  This method strips all subinterface information in the returned value.
+        """Return a tuple of numbers representing card, slot, port for this interface.  This method strips all subinterface information in the returned value.  If you call ordinal_list on GigabitEthernet2/25.100, you'll get this python tuple of integers: (2, 25).  If you call ordinal_list on GigabitEthernet2/0/25.100 you'll get this python list of integers: (2, 0, 25).
 
         Returns:
             - tuple.  A tuple of port numbers as integers.
@@ -676,7 +679,7 @@ class BaseNXOSIntfLine(NXOSCfgLine):
     @property
     def has_mpls(self):
         retval = self.re_match_iter_typed(
-            r'^\s*(mpls\s+ip)$', result_type=bool, default=False)
+            r'^\s*(mpls\s+ip\s+forwarding)$', result_type=bool, default=False)
         return retval
 
     @property
@@ -747,8 +750,10 @@ class BaseNXOSIntfLine(NXOSCfgLine):
             default=0.0)
         if (cd_seconds > 0.0):
             return cd_seconds
-        else:
+        elif (cd_msec > 0.0):
             return cd_msec / 1000.0
+        else:
+            return 0.0
 
     @property
     def has_manual_clock_rate(self):
@@ -1097,6 +1102,47 @@ class BaseNXOSIntfLine(NXOSCfgLine):
             r'^\s*arp\s+timeout\s+(\d+)\s*$', result_type=int, default=0)
         return retval
 
+    @property
+    def has_ip_helper_addresses(self):
+        """Return a True if the intf has helper-addresses; False if not"""
+        if len(self.ip_helper_addresses)>0:
+            return True
+        return False
+
+    @property
+    def ip_helper_addresses(self):
+        """Return a list of dicts with IP helper-addresses.  Each helper-address is in a dictionary.  The dictionary is in this format:
+
+        .. code-block:: python
+           :emphasize-lines: 11
+
+           >>> config = [
+           ...     '!',
+           ...     'interface Ethernet1/1',
+           ...     ' ip address 1.1.1.1/24',
+           ...     ' ip dhcp relay address 172.16.20.12',
+           ...     ' ip dhcp relay address 172.19.185.91',
+           ...     '!',
+           ...     ]
+           >>> parse = CiscoConfParse(config)
+           >>> obj = parse.find_objects('^interface\sEthernet1/1$')[0]
+           >>> obj.ip_helper_addresses
+           [{'addr': '172.16.20.12', 'vrf': '', 'global': False}, {'addr': '172.19.185.91', 'vrf': '', 'global': False}]
+           >>>"""
+        retval = list()
+
+        for child in self.children:
+            if 'vrf member' in child.text:
+                vrf = child.re_match_typed(r'vrf\s+member\s+(\S+)', default='')
+                break
+
+        for child in self.children:
+            if 'dhcp relay address' in child.text:
+                addr = child.re_match_typed(r'ip\s+dhcp\s+relay\s+address\s+(\d+\.\d+\.\d+\.\d+)')
+                global_addr = ''
+                retval.append({'addr': addr, 'vrf': vrf, 'global': bool(global_addr)})
+        return retval
+
     @property
     def is_switchport(self):
         retval = self.re_match_iter_typed(
@@ -1117,12 +1163,22 @@ class BaseNXOSIntfLine(NXOSCfgLine):
 
     @property
     def manual_switch_trunk_encap(self):
+        """Return a string with the switchport encapsulation type; if there is no manual trunk encapsulation, return ''."""
         retval = self.re_match_iter_typed(
-            r'^\s*(switchport\s+trunk\s+encap\s+(\S+))\s*$',
+            r'^\s*(switchport\s+trunk\s+encapsulation\s+(\S+))\s*$',
             result_type=str,
             default='')
         return retval
 
+    @property
+    def has_manual_switch_fex_fabric(self):
+        """Return a boolean indicating whether this port is configured in fex-fabric mode"""
+        retval = self.re_match_iter_typed(
+            r'^\s*(switchport\smode\s+fex-fabric)\s*$',
+            result_type=bool,
+            default=False)
+        return retval
+
     @property
     def has_manual_switch_trunk(self):
         retval = self.re_match_iter_typed(
@@ -1131,6 +1187,7 @@ class BaseNXOSIntfLine(NXOSCfgLine):
             default=False)
         return retval
 
+
     @property
     def has_switch_portsecurity(self):
         if not self.is_switchport:
@@ -1179,56 +1236,102 @@ class BaseNXOSIntfLine(NXOSCfgLine):
             default=default_val)
         return retval
 
+    @property
+    def manual_stp_link_type(self):
+        """Return a string with the spanning-tree link  type configured on this switchport; if there is no STP link type configured, return ''."""
+        retval = self.re_match_iter_typed(
+            r'^\s*spanning-tree\s+link-type\s+(\S.+?)$',
+            result_type=str,
+            default='')
+        return retval
+
+    @property
+    def manual_stp_port_type(self):
+        """Return a string with the spanning-tree port type configured on this switchport; if there is no STP port type configured, return '' (by default NXOS assigns this as 'normal', but this property is for a *manual* assignment)."""
+        retval = self.re_match_iter_typed(
+            r'^\s*spanning-tree\s+port\s+type\s+(\S.+?)$',
+            result_type=str,
+            default='')
+        return retval
+
+    @property
+    def vpc(self):
+        """Return an integer with the vpc id; Return 0 if there is no vpc id on this port"""
+        retval = self.re_match_iter_typed(
+            r'^\s*vpc\s+(\d+)$',
+            result_type=int,
+            default=0)
+        return retval
+
+    @property
+    def fex_associate_chassis_id(self):
+        """Return an integer with the fex chassis-id, return 0 if there is no 'fex associate' command on this switchport"""
+        retval = self.re_match_iter_typed(
+            r'^\s*fex\s+associate\s+(\d+)$',
+            result_type=int,
+            default=0)
+        return retval
+
     @property
     def trunk_vlans_allowed(self):
-        """Return a CiscoRange() with the list of allowed vlan numbers.  Return 0 if the port isn't a switchport"""
-        if self.is_switchport:
-            default_val = '1-4094'
+        """Return a CiscoRange() with the list of allowed vlan numbers (as int).  Return 0 if the port isn't a switchport in trunk mode"""
+
+        # The default values...
+        if self.is_switchport and not self.has_manual_switch_access:
+            retval = CiscoRange('1-{0}'.format(MAX_VLAN), result_type=int)
         else:
-            default_val = 0
+            return 0
 
-        allowed = self.re_match_iter_typed(
-            r'^\s*switchport\s+trunk\s+allowed\s+vlan\s+(\S+)$',
-            result_type=str,
-            default='1-4094')
-        if allowed == 'all':
-            allowed = '1-4094'
-        elif allowed == 'none':
-            allowed = ''
-        add = self.re_match_iter_typed(
-            r'^\s*switchport\s+trunk\s+allowed\s+vlan\s+add\s+(\S+)$',
-            result_type=str,
-            default='')
+        ## Iterate over switchport trunk statements
+        for obj in self.children:
 
-        if allowed and add:
-            combined = allowed + ',' + add
-        elif allowed:
-            combined = allowed
-        elif add:
-            combined = add
-        else:
-            combined = allowed
+            ## For every child object, check whether the vlan list is modified
+            abs_str = obj.re_match_typed(
+                '^\s+switchport\s+trunk\s+allowed\s+vlan\s(all|none|\d.*?)$',
+                default='_nomatch_', result_type=str).lower()
+            add_str = obj.re_match_typed(
+                '^\s+switchport\s+trunk\s+allowed\s+vlan\s+add\s+(\d.*?)$',
+                default='_nomatch_', result_type=str).lower()
+            exc_str = obj.re_match_typed(
+                '^\s+switchport\s+trunk\s+allowed\s+vlan\s+except\s+(\d.*?)$',
+                default='_nomatch_', result_type=str).lower()
+            rem_str = obj.re_match_typed(
+                '^\s+switchport\s+trunk\s+allowed\s+vlan\s+remove\s+(\d.*?)$',
+                default='_nomatch_', result_type=str).lower()
 
-        remove = self.re_match_iter_typed(
-            r'^\s*switchport\s+trunk\s+allowed\s+vlan\s+remove\s+(\S+)$',
-            result_type=str,
-            default='')
 
-        if remove:
-            retval = CiscoRange(combined, result_type=int).remove(remove)
-        else:
-            retval = CiscoRange(combined, result_type=int)
+            ## Build a vdict for each vlan modification statement
+            vdict = {
+                'absolute_str': abs_str,
+                'add_str': add_str,
+                'except_str': exc_str,
+                'remove_str': rem_str,
+            }
 
-        _except = self.re_match_iter_typed(
-            r'^\s*switchport\s+trunk\s+allowed\s+vlan\s+except\s+(\S+)$',
-            result_type=str,
-            default='')
-
-        if _except:
-            retval = CiscoRange(combined, result_type=int).remove(_except)
+            ## Analyze each vdict in sequence and apply to retval sequentially
+            for key, val in vdict.items():
+                if val!='_nomatch_':
+                    ## absolute in the key overrides previous values
+                    if 'absolute' in key:
+                        if val.lower()=='all':
+                            retval = CiscoRange('1-{0}'.format(MAX_VLAN),
+                                result_type=int)
+                        elif val.lower()=='none':
+                            retval = CiscoRange(result_type=int)
+                        else:
+                            retval = CiscoRange(val, result_type=int)
+                    elif 'add' in key:
+                        retval.append(val)
+                    elif 'except' in key:
+                        retval = CiscoRange('1-{0}'.format(MAX_VLAN),
+                            result_type=int)
+                        retval.remove(val)
+                    elif 'remove' in key:
+                        retval.remove(val)
 
         return retval
 
+
     @property
     def native_vlan(self):
         """Return an integer with the native vlan number.  Return 1, if the switchport has no explicit native vlan configured; return 0 if the port isn't a switchport"""
@@ -1273,17 +1376,19 @@ class BaseNXOSIntfLine(NXOSCfgLine):
         ## NOTE: I have no intention of checking self.is_shutdown here
         ##     People should be able to check the sanity of interfaces
         ##     before they put them into production
+        retval = ''
 
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
         if (self.ipv4_addr == ''):
             return ''
 
-        retval = self.re_match_iter_typed(
-            r'^\s*standby\s+(\d+\s+)*ip\s+(\S+)',
-            group=2,
-            result_type=str,
-            default='')
+        for hsrpobj in self.children:
+            if hsrpobj.re_match_typed(r'^\s+hsrp\s+(\d+)'):
+                for child in hsrpobj.children:
+                    retval = child.re_match_typed(r'^\s+ip\s+(\S+)')
+                    if retval: 
+                        return retval
         return retval
 
     @property
@@ -1291,6 +1396,7 @@ class BaseNXOSIntfLine(NXOSCfgLine):
         ## NOTE: I have no intention of checking self.is_shutdown here
         ##     People should be able to check the sanity of interfaces
         ##     before they put them into production
+        retval = ''
 
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
@@ -1307,43 +1413,78 @@ class BaseNXOSIntfLine(NXOSCfgLine):
     def hsrp_group(self):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
-        retval = self.re_match_iter_typed(
-            r'^\s*standby\s+(\d+)\s+ip\s+\S+', result_type=int, default=-1)
+        retval = ''
+        for hsrpobj in self.children:
+            retval = hsrpobj.re_match_typed(r'^\s+hsrp\s+(\d+)')
+            if retval:
+                return retval
         return retval
 
     @property
     def hsrp_priority(self):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
+        DEFAULT_PRI = 100
         if not self.has_ip_hsrp:
             return 0  # Return this if there is no hsrp on the interface
-        retval = self.re_match_iter_typed(
-            r'^\s*standby\s+(\d+\s+)*priority\s+(\d+)',
-            group=2,
-            result_type=int,
-            default=100)
-        return retval
+        for hsrpobj in self.children:
+            if hsrpobj.re_match_typed(r'^\s+hsrp\s+(\d+)'):
+                retval = hsrpobj.re_match_iter_typed(r'^\s+priority\s+(\d+)',
+                    result_type=int,
+                    default=DEFAULT_PRI)
+                if retval!=DEFAULT_PRI:
+                    return retval
 
     @property
     def hsrp_hello_timer(self):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
-        retval = self.re_match_iter_typed(
-            r'^\s*standby\s+(\d+\s+)*timers\s+(\d+)\s+\d+',
-            group=2,
-            result_type=int,
-            default=0)
+
+        #timers msec 250 msec 750
+        for hsrpobj in self.children:
+            if hsrpobj.re_match_typed(r'^\s+hsrp\s+(\d+)'):
+                timer_sec = hsrpobj.re_match_iter_typed(
+                    r'^\s+timers\s+(\d+)\s+\d+',
+                    result_type=float,
+                    default=0.0
+                    )
+                timer_msec = hsrpobj.re_match_iter_typed(
+                    r'^\s+timers\s+msec\s+(\d+)\s+msec\s+\d+',
+                    result_type=float,
+                    default=0.0
+                    )
+                if (timer_sec > 0.0):
+                    return timer_sec
+                elif (timer_msec > 0.0):
+                    return timer_msec / 1000.0
+                return 0.0
+
         return retval
 
     @property
     def hsrp_hold_timer(self):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
-        retval = self.re_match_iter_typed(
-            r'^\s*standby\s+(\d+\s+)*timers\s+\d+\s+(\d+)',
-            group=2,
-            result_type=int,
-            default=0)
+
+        #timers msec 250 msec 750
+        for hsrpobj in self.children:
+            if hsrpobj.re_match_typed(r'^\s+hsrp\s+(\d+)'):
+                timer_sec = hsrpobj.re_match_iter_typed(
+                    r'^\s+timers\s+\d+\s+(\d+)',
+                    result_type=float,
+                    default=0.0
+                    )
+                timer_msec = hsrpobj.re_match_iter_typed(
+                    r'^\s+timers\s+msec\s+\d+\s+msec\s+(\d+)',
+                    result_type=float,
+                    default=0.0
+                    )
+                if (timer_sec > 0.0):
+                    return timer_sec
+                elif (timer_msec > 0.0):
+                    return timer_msec / 1000.0
+                return 0.0
+
         return retval
 
     @property
@@ -1354,11 +1495,12 @@ class BaseNXOSIntfLine(NXOSCfgLine):
     def hsrp_track(self):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
-        retval = self.re_match_iter_typed(
-            r'^\s*standby\s+(\d+\s+)*track\s(\S+.+?)\s+\d+\s*',
-            group=2,
-            result_type=str,
-            default='')
+        retval = ''
+        for hsrpobj in self.children:
+            if hsrpobj.re_match_typed(r'^\s+hsrp\s+(\d+)'):
+                retval = hsrpobj.re_match_iter_typed(r'^\s+track\s+(\d+)',
+                    result_type=str,
+                    default='')
         return retval
 
     @property
@@ -1366,8 +1508,8 @@ class BaseNXOSIntfLine(NXOSCfgLine):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
         retval = self.re_match_iter_typed(
-            r'^\s*standby\s+(\d+\s+)*(use-bia)',
-            group=2,
+            r'^\s*hsrp\s+(use-bia)',
+            group=1,
             result_type=bool,
             default=False)
         return retval
@@ -1376,15 +1518,19 @@ class BaseNXOSIntfLine(NXOSCfgLine):
     def has_hsrp_preempt(self):
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
-        retval = self.re_match_iter_typed(
-            r'^\s*standby\s+(\d+\s+)*(use-bia)',
-            group=2,
-            result_type=bool,
-            default=False)
+        retval = False
+        for hsrpobj in self.children:
+            if hsrpobj.re_match_typed(r'^\s+hsrp\s+(\d+)'):
+                retval = hsrpobj.re_match_iter_typed(
+                    r'^\s+(preempt)',
+                    group=1,
+                    result_type=bool,
+                    default=False)
         return retval
 
     @property
     def hsrp_authentication_md5_keychain(self):
+        ## FIXME nxos
         ## For API simplicity, I always assume there is only one hsrp 
         ##     group on the interface
         retval = self.re_match_iter_typed(
@@ -1396,6 +1542,7 @@ class BaseNXOSIntfLine(NXOSCfgLine):
 
     @property
     def has_hsrp_authentication_md5(self):
+        ## FIXME nxos
         keychain = self.hsrp_authentication_md5_keychain
         return bool(keychain)
 
@@ -1491,17 +1638,17 @@ class NXOSIntfLine(BaseNXOSIntfLine):
           All :class:`~models_nxos.NXOSIntfLine` methods are still considered beta-quality, until this notice is removed.  The behavior of APIs on this object could change at any time.
         """
         super(NXOSIntfLine, self).__init__(*args, **kwargs)
+        self.feature = 'interface'
 
     @classmethod
     def is_object_for(cls, line="", re=re):
-        intf_regex = r'^interface\s+(\S+.+)'
-        if re.search(intf_regex, line):
+        if re.search(r'^interface\s+(\S.+)', line):
             return True
         return False
 
 
 ##
-##-------------  IOS Interface Globals
+##-------------  NXOS Interface Globals
 ##
 
 
@@ -1533,12 +1680,6 @@ class NXOSIntfGlobal(BaseCfgLine):
             return True
         return False
 
-    @property
-    def has_stp_portfast_def(self):
-        if self.re_search('^spanning-tree\sportfast\sdefault'):
-            return True
-        return False
-
     @property
     def has_stp_portfast_bpduguard_def(self):
         if self.re_search('^spanning-tree\sportfast\sbpduguard\sdefault'):
@@ -1551,9 +1692,139 @@ class NXOSIntfGlobal(BaseCfgLine):
             return True
         return False
 
+##
+##-------------  NXOS vPC line
+##
+class NXOSvPCLine(BaseCfgLine):
+    def __init__(self, *args, **kwargs):
+        super(NXOSvPCLine, self).__init__(*args, **kwargs)
+        self.feature = 'vpc'
+
+    def __repr__(self):
+        return "<%s # %s '%s'>" % (self.classname, self.linenum, self.vpc_domain_id)
+
+    @classmethod
+    def is_object_for(cls, line="", re=re):
+        if re.search(r'^vpc\s+domain', line):
+            return True
+        return False
+
+    @property
+    def vpc_domain_id(self):
+        retval = self.re_match_typed(r'^vpc\s+domain\s+(\d+)$',
+            result_type=str, default=-1)
+        return retval
+
+    @property
+    def vpc_role_priority(self):
+        retval = self.re_match_iter_typed(r'^\s+role\s+priority\s+(\d+)',
+            result_type=int, default=-1)
+        return retval
+
+    @property
+    def vpc_system_priority(self):
+        retval = self.re_match_iter_typed(r'^\s+system-priority\s+(\d+)',
+            result_type=int, default=-1)
+        return retval
+
+    @property
+    def vpc_system_mac(self):
+        retval = self.re_match_iter_typed(r'^\s+system-mac\s+(\S+)',
+            result_type=str, default='')
+        return retval
+
+    @property
+    def has_peer_config_check_bypass(self):
+        retval = self.re_match_iter_typed(r'^\s+(peer-config-check-bypass)',
+            result_type=bool, default=False)
+        return retval
+
+    @property
+    def has_peer_switch(self):
+        retval = self.re_match_iter_typed(r'^\s+(peer-switch)',
+            result_type=bool, default=False)
+        return retval
+
+    @property
+    def has_layer3_peer_router(self):
+        retval = self.re_match_iter_typed(r'^\s+(layer3\s+peer-router)',
+            result_type=bool, default=False)
+        return retval
+
+    @property
+    def has_peer_gateway(self):
+        retval = self.re_match_iter_typed(r'^\s+(peer-gateway)',
+            result_type=bool, default=False)
+        return retval
+
+    @property
+    def has_auto_recovery(self):
+        retval = self.re_match_iter_typed(r'^\s+(auto-recovery)',
+            result_type=bool, default=False)
+        return retval
+
+    @property
+    def vpc_auto_recovery_reload_delay(self):
+        reload_delay_regex = r'^\s+auto-recovery\s+reload-delay\s+(\d+)'
+        retval = self.re_match_iter_typed(reload_delay_regex,
+            result_type=int, default=-1)
+        return retval
+
+    @property
+    def has_ip_arp_synchronize(self):
+        retval = self.re_match_iter_typed(r'(ip\s+arp\s+synchronize)',
+            result_type=bool, default=False)
+        return retval
+
+    @property
+    def vpc_peer_keepalive(self):
+        """Return a dictionary with the configured vPC peer-keepalive parameters"""
+        dest = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?destination\s+(\d+\.\d+\.\d+\.\d+)',
+            result_type=str, default="")
+        hold_timeout = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?hold-timeout\s+(\d+)',
+            result_type=int, default=-1)
+        interval = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?interval\s+(\d+)',
+            result_type=int, default=-1)
+        timeout = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?timeout\s+(\d+)',
+            result_type=int, default=-1)
+        prec = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?precedence\s+(\S+)',
+            result_type=str, default="")
+        source = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?source\s+(\d+\.\d+\.\d+\.\d+)',
+            result_type=str, default="")
+        tos = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?tos\s+(\S+)',
+            result_type=str, default="")
+        tos_byte = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?tos-byte\s+(\S+)',
+            result_type=int, default=-1)
+        udp_port = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?udp-port\s+(\d+)',
+            result_type=int, default=-1)
+        vrf = self.re_match_iter_typed(
+            r'peer-keepalive\s+.*?vrf\s+(\S+)',
+            result_type=str, default="")
+        retval = {
+            'destination': dest,
+            'hold-timeout': hold_timeout,
+            'interval': interval,
+            'timeout': timeout,
+            'precedence': prec,
+            'source': source,
+            'tos': tos,
+            'tos-byte': tos_byte,
+            'udp-port': udp_port,
+            'vrf': vrf,
+            }
+        return retval
 
 ##
-##-------------  IOS Hostname Line
+##-------------  NXOS Hostname Line
 ##
 
 
@@ -1579,7 +1850,7 @@ class NXOSHostnameLine(BaseCfgLine):
 
 
 ##
-##-------------  IOS Access Line
+##-------------  NXOS Access Line
 ##
 
 
@@ -1660,7 +1931,7 @@ class NXOSAccessLine(BaseCfgLine):
 
 
 ##
-##-------------  Base IOS Route line object
+##-------------  Base NXOS Route line object
 ##
 
 
@@ -1716,7 +1987,7 @@ class BaseNXOSRouteLine(BaseCfgLine):
 
 
 ##
-##-------------  IOS Route line object
+##-------------  NXOS Route line object
 ##
 
 _RE_IP_ROUTE = re.compile(r"""^ip\s+route
@@ -1892,7 +2163,7 @@ class NXOSRouteLine(BaseNXOSRouteLine):
 
 
 ##
-##-------------  IOS TACACS+ Group
+##-------------  NXOS TACACS+ Group
 ##
 class NXOSAaaGroupServerLine(BaseCfgLine):
     def __init__(self, *args, **kwargs):
@@ -1942,7 +2213,7 @@ class NXOSAaaGroupServerLine(BaseCfgLine):
 
 
 ##
-##-------------  IOS AAA Lines
+##-------------  NXOS AAA Lines
 ##
 
 
